// SilentTimer Action Figure communication library
// libstaf.h, v1.0
// 
// 03.07.2020
// Copyright (C) 2020 Balthasar Szczepański
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <stdint.h>

//serial communication errors
#define ERR_DATA_TOO_LONG     1
#define ERR_NO_DATA           2
#define ERR_INCOMPLETE_DATA   3
#define ERR_INVALID_DATA      4
//command errors
#define ERR_MISSING_POINTER   5
#define ERR_INVALID_PARAMETER 6

//"present" modes:
#define mode_time        0x00
#define mode_alarm       0x01
#define mode_date        0x02
#define mode_countup     0x03
#define mode_countdown   0x04
#define mode_calibration 0x05
#define mode_version     0x06
#define mode_debug       0x07
#define mode_reset       0x08
#define mode_calculator  0x09
//"edit" modes:
#define mode_set_time          0x80
#define mode_set_alarm         0x81
#define mode_set_date          0x82
#define mode_set_year          0x83
#define mode_set_countdown     0x84
#define mode_set_calibration   0x85
#define mode_set_debug_address 0x86
#define mode_set_debug_value   0x87
#define mode_set_calc_number   0x88
#define mode_set_calc_operator 0x89

//triggerable events:
#define event_tick      0x01
#define event_button_a  0x02
#define event_button_b  0x04
#define event_button_c  0x08
//non-triggerable events
#define event_alarm     0x20
#define event_countdown 0x40

#define event_mask      0x6f

//calculator operations
#define calc_add      0x00
#define calc_subtract 0x40
#define calc_multiply 0x80
#define calc_divide   0xc0

#define calc_mask     0xc0

//buzzer modes
#define buzzer_enabled  0
#define buzzer_disabled 1
#define buzzer_forced   2


//display segments
#define segment_a0 0x00000001
#define segment_b0 0x00000002
#define segment_c0 0x00000004
#define segment_d0 0x00000008
#define segment_e0 0x00000010
#define segment_f0 0x00000020
#define segment_g0 0x00000040
#define segment_h0 0x00000080

#define segment_a1 0x00000100
#define segment_b1 0x00000200
#define segment_c1 0x00000400
#define segment_d1 0x00000800
#define segment_e1 0x00001000
#define segment_f1 0x00002000
#define segment_g1 0x00004000
#define segment_h1 0x00008000

#define segment_a2 0x00010000
#define segment_b2 0x00020000
#define segment_c2 0x00040000
#define segment_d2 0x00080000
#define segment_e2 0x00100000
#define segment_f2 0x00200000
#define segment_g2 0x00400000
#define segment_h2 0x00800000

#define segment_a3 0x01000000
#define segment_b3 0x02000000
#define segment_c3 0x04000000
#define segment_d3 0x08000000
#define segment_e3 0x10000000
#define segment_f3 0x20000000
#define segment_g3 0x40000000
#define segment_h3 0x80000000

#define segment_n0   segment_a0
#define segment_ne0  segment_b0
#define segment_se0  segment_c0
#define segment_s0   segment_d0
#define segment_sw0  segment_e0
#define segment_nw0  segment_f0
#define segment_mid0 segment_g0

#define segment_n1   segment_a1
#define segment_ne1  segment_b1
#define segment_se1  segment_c1
#define segment_s1   segment_d1
#define segment_sw1  segment_e1
#define segment_nw1  segment_f1
#define segment_mid1 segment_g1

#define segment_n2   segment_a2
#define segment_ne2  segment_b2
#define segment_se2  segment_c2
#define segment_s2   segment_d2
#define segment_sw2  segment_e2
#define segment_nw2  segment_f2
#define segment_mid2 segment_g2

#define segment_n3   segment_a3
#define segment_ne3  segment_b3
#define segment_se3  segment_c3
#define segment_s3   segment_d3
#define segment_sw3  segment_e3
#define segment_nw3  segment_f3
#define segment_mid3 segment_g3

#define dot_top      segment_h0
#define dot_bottom   segment_h1
#define dot_alarm    segment_h3


struct StafState
{
	uint8_t mode;
	uint8_t running;
	uint8_t digit;
	uint8_t value[3];
};


int stafConnect (
	const char *port
);

int stafDisconnect (
	int fd
);


int stafGetState (
	int fd,
	uint8_t *event,
	struct StafState *state
);

int stafSetState (
	int fd,
	uint8_t *event,
	struct StafState *state
);


int stafGetMode (
	int fd,
	uint8_t *event,
	uint8_t *mode
);

int stafSetMode (
	int fd,
	uint8_t *event,
	uint8_t mode
);


int stafCheckEvent (
	int fd,
	uint8_t *event
);

int stafSubscribeEvent (
	int fd, uint8_t *event,
	uint8_t subscribeEvent
);

int stafTriggerEvent (
	int fd,
	uint8_t *event,
	uint8_t triggerEvent
);


int stafGetPinout (
	int fd,
	uint8_t *event,
	uint8_t *pinout //8 bytes
);

int stafGetDisplay (
	int fd,
	uint8_t *event,
	uint8_t *pinout, //8 bytes
	uint32_t *display
);

int stafSetDisplay (
	int fd,
	uint8_t *event,
	uint8_t *pinout, //8 bytes
	uint32_t display
);

int stafFreeDisplay (
	int fd,
	uint8_t *event
);


int stafGetTime (
 int fd,
 uint8_t *event,
 uint8_t *cs,
 uint8_t *second,
 uint8_t *minute,
 uint8_t *hour,
 uint8_t *day,
 uint8_t *month,
 uint16_t *year
);

int stafSetTime (
	int fd,
	uint8_t *event,
	uint8_t minute,
	uint8_t hour,
	uint8_t restoreState
);

int stafSetDate (
	int fd,
	uint8_t *event,
	uint8_t day,
	uint8_t month,
	uint8_t restoreState
);

int stafSetYear (
	int fd,
	uint8_t *event,
	uint16_t year,
	uint8_t restoreState
);


int stafGetAlarm (
	int fd,
	uint8_t *event,
	uint8_t *enabled,
	uint8_t *minute,
	uint8_t *hour
);

int stafSetAlarm (
	int fd,
	uint8_t *event,
	uint8_t minute,
	uint8_t hour,
	uint8_t restoreState
);

int stafClearAlarm (
	int fd,
	uint8_t *event,
	uint8_t restoreState
);


int stafSetCountdown (
	int fd,
	uint8_t *event,
	uint8_t second,
	uint8_t minute,
	uint8_t restoreState
);

int stafIsRunning (
	int fd,
	uint8_t *event,
	uint8_t *running
);

int stafSetRunning (
	int fd,
	uint8_t *event,
	uint8_t running
);


int stafGetVersion (
	int fd, uint8_t *event,
	uint8_t *minor,
	uint8_t *major
);


int stafGetDigit (
	int fd,
	uint8_t *event,
	uint8_t *digit
);

int stafSetDigit (
	int fd,
	uint8_t *event,
	uint8_t digit
);

int stafGetDigitValue (
	int fd,
	uint8_t *event,
	uint8_t *value
);

int stafSetDigitValue (
	int fd,
	uint8_t *event,
	uint8_t value
);

int stafGetValue (
	int fd,
	uint8_t *event, 
	uint8_t *value //3 bytes
);

int stafSetValue (
	int fd,
	uint8_t *event,
	uint8_t *value //3 bytes
);

int stafConfirm (
	int fd,
	uint8_t *event
);


int stafSetDebugAddress (
	int fd,
	uint8_t *event,
	uint8_t address
);

int stafGetDebugValue (
	int fd,
	uint8_t *event,
	uint8_t address,
	uint8_t *value,
	uint8_t restoreState
);

int stafSetDebugValue (
	int fd,
	uint8_t *event,
	uint8_t address,
	uint8_t value,
	uint8_t restoreState
);

int stafReadMemory (
	int fd,
	uint8_t *event,
	uint16_t address,
	uint8_t *value
);

int stafWriteMemory (
	int fd,
	uint8_t *event,
	uint16_t address,
	uint8_t value
);

int stafSetCalibration (
	int fd,
	uint8_t *event,
	int16_t calibration,
	uint8_t restoreState
);

int stafGetCalibration (
	int fd,
	uint8_t *event,
	int16_t *calibration,
	uint8_t restoreState
);


int stafSetBuzzerMode (
	int fd,
	uint8_t *event,
	uint8_t mode
);


int stafCalculate (
	int fd,
	uint8_t *event,
	float a,
	float b,
	uint8_t operation,
	float *y,
	uint8_t *error,
	uint8_t restoreState
);
